'use strict';

var _require = require('chai'),
    expect = _require.expect;

var bunyan = require('bunyan');
var swatch = require('swatchjs');

var batch = require('../lib/batch');

var logger = bunyan.createLogger({
  name: 'swatch-batch-koa-test',
  streams: [{ path: '/dev/null' }]
});

var onException = function onException(errorObj) {
  throw errorObj;
};

var swatchCtx = {
  logger: logger,
  request: {
    onException: onException
  }
};

describe('batch', function () {
  it('should return a handler function', function () {
    var handler = batch([]);
    expect(handler).to.be.a('function');
  });

  it('should throw if parameter is not a model', function () {
    expect(function () {
      return batch(10);
    }).to.throw();
  });
});

describe('batch handler', function () {
  var model = swatch({
    add: function add(a, b) {
      return a + b;
    },
    sub: function sub(a, b) {
      return a - b;
    },
    bad: function bad() {
      throw new Error('some_error');
    },
    mid: {
      handler: function handler() {
        throw new Error('handler_exception');
      },
      metadata: {
        middleware: [function () {
          throw new Error('middleware_exception');
        }]
      }
    }
  });
  var handler = batch(model);

  it('should execute all requested methods', function (done) {
    var ops = [{
      method: 'add',
      args: {
        a: 10,
        b: 20
      }
    }, {
      method: 'sub',
      args: {
        a: 50,
        b: 25
      }
    }];
    var expected = [{
      ok: true,
      result: 30
    }, {
      ok: true,
      result: 25
    }];

    handler.call(swatchCtx, ops).then(function (result) {
      expect(result).to.deep.equal(expected);
      done();
    }).catch(done);
  });

  it('should return an error if a method does not exist', function (done) {
    var ops = [{
      method: 'add',
      args: {
        a: 10,
        b: 20
      }
    }, {
      method: 'foo',
      args: {}
    }];
    var expected = [{
      ok: true,
      result: 30
    }, {
      ok: false,
      error: 'invalid_method',
      details: undefined
    }];

    handler.call(swatchCtx, ops).then(function (result) {
      expect(result).to.deep.equal(expected);
      done();
    }).catch(done);
  });

  it('should return an error if the method throws', function (done) {
    var ops = [{
      method: 'add',
      args: {
        a: 10,
        b: 20
      }
    }, {
      method: 'bad',
      args: {}
    }];
    var expected = [{
      ok: true,
      result: 30
    }, {
      ok: false,
      error: 'some_error',
      details: undefined
    }];

    handler.call(swatchCtx, ops).then(function (result) {
      expect(result).to.deep.equal(expected);
      done();
    }).catch(done);
  });

  it('should run middleware before each method', function (done) {
    var ops = [{
      method: 'add',
      args: {
        a: 10,
        b: 20
      }
    }, {
      method: 'mid',
      args: {}
    }];
    var expected = [{
      ok: true,
      result: 30
    }, {
      ok: false,
      error: 'middleware_exception',
      details: undefined
    }];

    handler.call(swatchCtx, ops).then(function (result) {
      expect(result).to.deep.equal(expected);
      done();
    }).catch(done);
  });

  it('should throw if ops is invalid', function (done) {
    handler(10).then(function () {
      done(new Error('invalid_ops_should_have_failed'));
    }).catch(function (error) {
      expect(error.message).to.equal('invalid_request');
      done();
    });
  });
});